@@ -3,11 +3,11 @@
 """ The work time log app tracks working hours in an interaction model similar to git. """
 
 import csv
-from dataclasses import dataclass
 import datetime as dt
 import os
 import sys
-from typing import Callable, Dict, List, Optional, Tuple
+from dataclasses import dataclass
+from typing import Callable, Dict, List, Optional, Set, Tuple
 
 from work_components import consts, dt_parse, util
 from work_components.arguments import (
@@ -22,11 +22,12 @@ from work_components.consts import DIRECTORY, DIRECTORY_DEBUG
 from work_components.container import ProtocolMeta, Record, sort_and_merge
 from work_components.rc_dao import RC
 from work_components.recess_dao import RecessDao
+from work_components.util import Color, PrinTable
 from work_components.work_dao import WorkDao
 
 assert sys.version_info >= (3, 8)
 
-VERSION: str = "0.97.3"
+VERSION: str = "0.98.1"
 
 # Formats
 TIME_FORMAT = "%H:%M"
@@ -635,13 +636,18 @@ class Work(object):
         minutes_logged, minutes_active_run = self._total_minutes_worked_today()
         new_section = lambda: "\n> "
 
-        output: str = "{} on record, {} active run".format(
+        header: str = "{} on record, {} active run".format(
             self._timedelta_str(minutes_logged),
             self._timedelta_str(minutes_active_run),
         )
 
+        balance_message = self._get_day_balance_msg(minutes_logged + minutes_active_run)
+        header += f" | {balance_message}"
+
         ## Optional modes
 
+        optional_output: str = ""
+
         start_time: Optional[dt.datetime] = self.dao.get_start_time()
 
         # prospective_start | If start_time is None, no run is active. Then we assume
@@ -652,7 +658,22 @@ class Work(object):
             rounding_mode=dt_parse.RoundingMode.DOWN,
         )
 
-        # worked_towork_split | By default, we base on the prospective start time.
+        start_argument: bool = args.h_start is not None
+        if start_argument:
+            # Time calculation incorporates minutes worked in the active run so far.
+            # Therefore, we can't override the active run start time.
+            if start_time is not None:
+                raise InvalidOperationWarning(
+                    "Flag --start may only be used if no run is active",
+                )
+            prospective_start = dt_parse.resolve_time_argument(
+                args.h_start,
+                baseline_date=dt.date.today(),
+                rounding_mode=dt_parse.RoundingMode.NONE,
+            )
+
+        # worked_towork_split | The time where the worked_minutes end.
+        #                       By default, we base on the prospective start time.
         #                       If a run was started in the past, worked_minutes
         #                       includes the minutes up to now, so we have to remove
         #                       those by basing on _dt_now_stripped().
@@ -660,12 +681,20 @@ class Work(object):
         if start_time is not None and start_time < dt.datetime.now():
             worked_towork_split = self._dt_now_stripped()
 
+        # If requested by the user, an extra pause is added to the calculation.
+        extra_pause_minutes: float = 0.0
+        if args.h_pause:
+            pause_hours, pause_minutes = dt_parse.parse_time_period_str(args.h_pause)
+            extra_pause_minutes = pause_minutes + 60 * pause_hours
+
         hours_data: Work.HoursData = Work.HoursData(
-            start_time=start_time,
             minutes_logged=minutes_logged,
             minutes_active_run=minutes_active_run,
             prospective_start=prospective_start,
+            run_active=start_time is not None,
+            start_argument=start_argument,
             worked_towork_split=worked_towork_split,
+            extra_pause_minutes=extra_pause_minutes,
         )
 
         if args.h_until:
@@ -682,42 +711,48 @@ class Work(object):
                     f"(prospective) start time {self._readable_dt(prospective_start)}."
                 )
 
-            output += new_section()
-            output += self._get_hours_worked_until_msg(
+            optional_output += new_section()
+            optional_output += self._get_hours_worked_until_msg(
                 target_time=target, hours_data=hours_data
             )
 
         # Only continue if args.h_target is set and not 0
         if args.h_target:
-            time: dt.time = dt_parse.parse_time_str(str(args.h_target))
+            target_hours, target_minutes = dt_parse.parse_time_period_str(args.h_target)
 
-            output += new_section()
-            output += self._get_target_end_time_msg(
-                target_hours=time.hour,
-                target_minutes=time.minute,
+            optional_output += new_section()
+            optional_output += self._get_target_end_time_msg(
+                target_hours=target_hours,
+                target_minutes=target_minutes,
                 hours_data=hours_data,
             )
 
         # Calculate the remaining hours for a full workday (respecting the week balance)
         if args.h_workday:
-            output += new_section()
-            output += self._get_workday_end_time_msg(hours_data=hours_data)
+            optional_output += new_section()
+            optional_output += self._get_workday_end_time_msg(hours_data=hours_data)
 
-        if args.h_balance:
-            output += new_section()
-            output += self._get_day_balance_msg(hours_data=hours_data)
+        # If any of the above routines have added to the output, we need a note.
+        note_str: str = ""
+        if optional_output:
+            assumption_str: str = self._build_hours_assumption_str(
+                hours_data=hours_data
+            )
+            note_str = "\n\n{}:".format(assumption_str) if assumption_str else ""
 
-        print(output)
+        print(f"{header}{note_str}{optional_output}")
 
     @dataclass
     class HoursData:
         """ Storage container for data used in hours sub-functions. """
 
-        start_time: Optional[dt.datetime]
         minutes_logged: float
         minutes_active_run: float
         prospective_start: dt.datetime
+        run_active: bool
+        start_argument: bool
         worked_towork_split: dt.datetime
+        extra_pause_minutes: float
 
         @property
         def total_minutes_worked(self) -> float:
@@ -732,27 +767,20 @@ class Work(object):
         ) / dt.timedelta(minutes=1)
         total_minutes: float = hours_data.total_minutes_worked + minutes_from_now
 
-        target_strf: str = target_time.strftime(TIME_FORMAT)
-        minutes_strf: str = self._timedelta_str(minutes=total_minutes)
+        if hours_data.extra_pause_minutes > 0:
+            total_minutes -= hours_data.extra_pause_minutes
 
-        if hours_data.start_time is None:
-            return (
-                f"You will have worked {minutes_strf} at target time {target_strf} if "
-                f"you start now ({hours_data.prospective_start.strftime('%H:%M')})"
-            )
-
-        return f"You will have worked {minutes_strf} at target time {target_strf}."
+        return (
+            f"You will have worked {self._timedelta_str(minutes=total_minutes)} "
+            f"at target time {target_time.strftime(TIME_FORMAT)}."
+        )
 
     def _get_target_end_time_msg(self, target_hours, target_minutes, hours_data):
-        # type: (float, float, Work.HoursData) -> str
+        # type: (int, int, Work.HoursData) -> str
         """ Compute and stringify the status towards a given time target. """
 
         target_time: str = f"{target_hours}:{target_minutes:0>2}"
-        target_minutes += target_hours * 60.0
-
-        # Remove ".0" for full hours
-        if int(target_hours) == target_hours:
-            target_hours = int(target_hours)
+        target_minutes += target_hours * 60
 
         return self._get_end_time_msg(
             target_minutes=target_minutes,
@@ -795,7 +823,7 @@ class Work(object):
         # type: (float, Work.HoursData, str, str) -> str
         """
         Compute the end time for a day of the specified hours.
-        Returns: None if target is achieved.
+        Returns: Compiled message for printing.
         """
 
         # If the expected hours were already reached before this run, we can immediately return.
@@ -804,6 +832,9 @@ class Work(object):
 
         delta_minutes: float = target_minutes - hours_data.total_minutes_worked
 
+        if hours_data.extra_pause_minutes > 0:
+            delta_minutes += hours_data.extra_pause_minutes
+
         # If a run was started in the past, worked_minutes includes the minutes up to now,
         # so we have to calculate the end from _dt_now_stripped().
         prospective_end: dt.datetime = hours_data.worked_towork_split
@@ -819,42 +850,60 @@ class Work(object):
 
         # End time not yet reached
 
-        message: str
-        if not self.dao.run_active():
-            message = f"Start now ({hours_data.prospective_start.strftime('%H:%M')}) and work "
-        else:
-            message = "Work "
-        message += f"until {prospective_end.strftime(TIME_FORMAT)} "
-
-        # Explainer, such as "for a 7 hour day"
-        message += target_explainer.strip()
-        message += "."
+        message: str = (
+            f"Work until {prospective_end.strftime(TIME_FORMAT)} "
+            f"{target_explainer.strip()}."  # explainer, such as "for a 7 hour day"
+        )
 
         return message
 
-    def _get_day_balance_msg(self, hours_data):
-        # type: (Work.HoursData) -> str
+    def _get_day_balance_msg(self, total_minutes_worked: float) -> str:
         """Compute and stringify the week balance including the current workday,
         taking into account all worked hours."""
 
         minutes_to_work: float = self._minutes_per_day(
             day=dt.date.today()
         ) + self._minute_balance_up_to(day=dt.date.today())
-        remaining_minutes: float = minutes_to_work - hours_data.total_minutes_worked
+        remaining_minutes: float = minutes_to_work - total_minutes_worked
 
-        message = "Current balance: "
-        timedelta_str: str = self._timedelta_str(abs(remaining_minutes))
+        message: str = "Balance: "
 
-        if remaining_minutes < 0:
-            message += timedelta_str + " overtime"
-        elif remaining_minutes > 0:
-            message += timedelta_str + " remaining"
-        else:
+        remaining_timedelta: str = self._timedelta_str(abs(remaining_minutes))
+        rm_postfix: str = "remaining" if remaining_minutes > 0 else "overtime"
+        if remaining_minutes == 0:
             message += "+/- 0"
+        else:
+            message += f"{remaining_timedelta} {rm_postfix}"
 
-        message += f" ({self._timedelta_str(minutes_to_work)} total)"
+        min_to_work_timedelta: str = self._timedelta_str(abs(minutes_to_work))
+        mtw_postfix: str = "to work" if minutes_to_work > 0 else "overtime"
 
-        return message
+        return f"{message} ({min_to_work_timedelta} {mtw_postfix} today)"
+
+    def _build_hours_assumption_str(self, hours_data):
+        # type: (Work.HoursData) -> str
+        """ Build a string describing assumptions used in the calculation. """
+
+        note_str: str = ""
+
+        start_time_strf: str = hours_data.prospective_start.strftime("%H:%M")
+        if hours_data.start_argument:
+            note_str += f"you start at {start_time_strf}"
+        elif not hours_data.run_active:
+            note_str += f"you start now ({start_time_strf})"
+
+        if hours_data.extra_pause_minutes > 0:
+            if len(note_str) > 0:
+                note_str += " and"
+            else:
+                note_str += "you"
+            pause_str: str = self._timedelta_str(hours_data.extra_pause_minutes)
+            note_str += f" take {pause_str} of breaks"
+
+        if len(note_str) > 0:
+            note_str = f"Assuming {note_str}"
+
+        return note_str
 
     # list #
 
@@ -1022,7 +1071,7 @@ class Work(object):
 
         total_minutes: float = 0.0
         total_break_minutes: float = 0.0
-        prin_table: util.PrinTable = util.PrinTable()
+        prin_table: PrinTable = PrinTable()
 
         for i, record in enumerate(records):
             minutes: float = record.get_minutes()
@@ -1032,12 +1081,6 @@ class Work(object):
                 f" | {self._timedelta_str(minutes)}",
             ]
 
-            # Add break info after all but the last entry
-            if print_breaks:
-                break_str, break_length = self._break_str(records, i)
-                total_break_minutes += break_length
-                row.append(Color.color(break_str, Color.GRAY))
-
             # Add optional attributes
             if not only_time:
                 category: str = f"  ({record.category})" if record.category else " "
@@ -1046,18 +1089,24 @@ class Work(object):
 
             prin_table.add_row(row)
 
+            # Add break info after all but the last entry
+            if print_breaks:
+                break_str, break_length = self._break_str(records, i)
+                total_break_minutes += break_length
+                if break_length > 0:
+                    prin_table.add_row(["", Color.color(break_str, Color.GRAY)])
+
+        # Replace time of active run with start time only, colored in blue.
         if active_start_to_include is not None:
-            # Note: If we were to just add a row to the table, the first column would
-            # be too wide (the color codes are not handled correctly by PrinTable).
-            # If we fix the color code handling, we can change this.
-            active_run_row: str = f"{active_start_to_include.strftime(TIME_FORMAT)} ~"
-            prin_table.rows[-1][0] = "{}{}".format(
-                Color.color(active_run_row, Color.BLUE),
-                " " * (13 - len(active_run_row)),
+            active_run_row: List[str] = prin_table.rows.pop()
+            active_run_time_str: str = (
+                f"{active_start_to_include.strftime(TIME_FORMAT)} ~"
             )
+            active_run_row[0] = Color.color(active_run_time_str, Color.BLUE)
+            prin_table.add_row(active_run_row)
 
         for row in prin_table.printable():
-            result.append("".join(row).strip())
+            result.append("".join(row).rstrip())
 
         if len(records) > 0:
             result[0] = Color.bold(result[0])
@@ -1080,7 +1129,7 @@ class Work(object):
 
         if break_length == 0:
             return "", 0
-        return f"  ~ {self._timedelta_str(break_length)} break", break_length
+        return f" ~ {self._timedelta_str(break_length)} break", break_length
 
     # day #
 
@@ -1241,20 +1290,39 @@ class Work(object):
             print(f"[{i}] {record.strf(TIME_FORMAT)}")
         print()
 
-        user_choice: str = ""
-        while user_choice not in ["all"] + [str(i) for i in all_indices]:
-            user_choice = input(
-                f'Which element? Enter an index [{0}..{len(records) - 1}] or "all": '
+        print(
+            f"Enter nothing to cancel, or\n"
+            f"Enter one or more indices [{0}..{len(records) - 1}] separated by a space, or\n"
+            f'Enter "all" to {verb} all entries.\n'
+        )
+        selected_indices: Set[int] = set()
+        while 0 == len(selected_indices):
+            user_choice: str = input(f"Which entries? > ")
+            if user_choice == "":
+                return None
+
+            selected_indices = self._get_index_selection(
+                user_choice=user_choice, valid_indices=all_indices
             )
 
-        selected_indices: List[int] = []
-        if user_choice == "all":
-            selected_indices = all_indices
-        else:
-            selected_indices = [int(user_choice)]
-
         return protocol, [zipped_records[i] for i in selected_indices]
 
+    def _get_index_selection(
+        self, user_choice: str, valid_indices: List[int]
+    ) -> Set[int]:
+        """ Match a user input to the chosen indices. Allows 1-many indices or "all". """
+        if user_choice == "all":
+            return set(valid_indices)
+        try:
+            selection: Set[int] = {int(u) for u in user_choice.split(" ") if len(u) > 0}
+        except ValueError:
+            print(" Not a number!")
+            return set()
+        if selection.difference(valid_indices):
+            print(" Invalid index chosen!")
+            return set()
+        return selection
+
     def _end_manipulation_mode(self, num_edited: int, verb: str) -> None:
         """ Print result message, update checksum. """
 
@@ -1644,31 +1712,6 @@ class Work(object):
 ### Other classes ###
 
 
-class Color(object):
-    """ See https://en.wikipedia.org/wiki/ANSI_escape_code#8-bit """
-
-    BLUE = 27
-    GRAY = 242
-    GREEN = 34
-    ORANGE = 202
-    RED = 9
-
-    @staticmethod
-    def color(text: str, clr_code: int, bg: bool = False) -> str:
-        fg_bg_code = "38" if not bg else "48"
-        return Color._format(
-            text=text, format_code="{};5;{}".format(fg_bg_code, clr_code)
-        )
-
-    @staticmethod
-    def bold(text: str) -> str:
-        return Color._format(text=text, format_code="1")
-
-    @staticmethod
-    def _format(text: str, format_code: str) -> str:
-        return "\x1b[{}m{}\x1b[0m".format(format_code, text)
-
-
 class InvalidOperationWarning(Warning):
     pass
 
