@@ -83,9 +83,8 @@ MODES: Dict[str, Mode] = {
         SWITCH_NAMES,
         help="Short-hand for stop & start",
         description=(
-            "A short-hand for stop and start. 'switch A B -c foo' runs 'stop A -c foo',"
-            " then 'start B'. 'switch A -c foo -m bar' runs 'stop A -c foo -m bar', then"
-            " 'start A'. Flags are passed to stop, meaning they refer to the stopped run."
+            "A short-hand for stop and start. Flags are passed to stop, meaning they "
+            "refer to the stopped run."
         ),
     ),
     CANCEL_NAME: Mode(CANCEL_NAME, help="Cancel the current run"),
@@ -255,17 +254,10 @@ class Arguments:
             dest="h_workday",
             help="Also show the end time of a complete workday with respect to the week balance.",
         )
-        hours_mode.add_argument(
-            "-b",
-            "--balance",
-            action="store_true",
-            dest="h_balance",
-            help="Also show the current week balance, including all hours worked today.",
-        )
         hours_mode.add_argument(
             "-u",
             "--until",
-            metavar="T",
+            metavar="H:M",
             dest="h_until",
             help="Also show the hours that will have been worked at the given time.",
         )
@@ -276,7 +268,6 @@ class Arguments:
             "--target",
             metavar="H:M",
             dest="h_target",
-            type=str,
             help=target_arg_help_text_stub
             + "a workday of the specified length in hours:minutes.",
         )
@@ -289,6 +280,21 @@ class Arguments:
             help=target_arg_help_text_stub
             + "an 8 hour day (equivalent to --target 8).",
         )
+        # Modifiers
+        hours_mode.add_argument(
+            "-p",
+            "--pause",
+            metavar="H:M",
+            dest="h_pause",
+            help="Assume for all calculations that work will be paused hours:minutes.",
+        )
+        hours_mode.add_argument(
+            "-s",
+            "--start",
+            metavar="H:M",
+            dest="h_start",
+            help="Override the start time assumed by hours. May only be used when no run is active.",
+        )
 
         # Parent parser for date ranges
         date_selection_parent = argparse.ArgumentParser(add_help=False)
