@@ -12,11 +12,11 @@ from typing import Iterable, List, Optional
 from work_components import util
 from work_components.consts import (
     DATETIME_FORMAT,
+    DAY_FILE_PATTERN,
+    MONTH_DIR_PATTERN,
+    PROTOCOL_FILE_EXTENSION,
     TIME_FORMAT,
     YEAR_DIR_PATTERN,
-    MONTH_DIR_PATTERN,
-    DAY_FILE_PATTERN,
-    PROTOCOL_FILE_EXTENSION,
 )
 
 ### Record container classes ###
@@ -59,6 +59,41 @@ class Record(object):
         """ Return the total minutes in this record. """
         return util.minutes_difference(start=self.start, end=self.end)
 
+    def overlaps(self, other) -> bool:
+        """ Check if the given `Record` overlaps this. """
+        # Either other starts within us or we start within other. For start, we allow
+        # equality (in case of an exact overlap), but for end we don't (touching is
+        # not overlapping).
+        return (
+            self.start <= other.start < self.end
+            or other.start <= self.start < other.end
+        )
+
+    def touches(self, other) -> bool:
+        """ Check if the given `Record` touches this. """
+        return self.start == other.end or self.end == other.start
+
+    def merge(self, other):
+        # type: (Record) -> Record
+        """
+        Try to merge two records. Can only merge touching or overlapping records that are otherwise identical.
+
+        Raises if the merge is unsuccessful.
+        """
+
+        if not (self.overlaps(other) or self.touches(other)):
+            raise Record.UnmergeableError("Time does not overlap or touch.")
+
+        if not (self.category == other.category and self.message == other.message):
+            raise Record.UnmergeableError("Category or message differ.")
+
+        return Record(
+            start=min((self.start, other.start)),
+            end=max((self.end, other.end)),
+            category=self.category,
+            message=self.message,
+        )
+
     def to_protocol_row(self) -> List[str]:
         """ Create a row for a protocol file from this record. """
         format = lambda d: d.strftime(DATETIME_FORMAT)
@@ -102,30 +137,6 @@ class Record(object):
             and self.message == other.message
         )
 
-    def merge(self, other):
-        # type: (Record) -> Record
-        """
-        Try to merge two records. Can only merge touching or overlapping records that are otherwise identical.
-
-        Raises if the merge is unsuccessful.
-        """
-
-        if not (
-            self.start <= other.start <= self.end
-            or other.start <= self.start <= other.end
-        ):
-            raise Record.UnmergeableError("Time does not overlap or touch.")
-
-        if not (self.category == other.category and self.message == other.message):
-            raise Record.UnmergeableError("Category or message differ.")
-
-        return Record(
-            start=min((self.start, other.start)),
-            end=max((self.end, other.end)),
-            category=self.category,
-            message=self.message,
-        )
-
     @staticmethod
     def from_protocol_row(row):
         # type: (List[str]) -> Record
@@ -142,9 +153,8 @@ class Record(object):
             )
 
         if len(row) != 4:
-            raise IOError(
-                f"Detected corrupted protocol row (expected 4 elements, got {len(row)}). "
-                "Please fix manually."
+            raise Record.CorruptedProtocolRowError(
+                f"expected 4 elements, got {len(row)}"
             )
 
         start: str
@@ -159,9 +169,15 @@ class Record(object):
         )
 
     class UnmergeableError(Exception):
-        def __init__(self, msg):
+        def __init__(self, msg: str) -> None:
             super().__init__(f"Unmergeable records: {msg}")
 
+    class CorruptedProtocolRowError(Exception):
+        def __init__(self, msg: str) -> None:
+            super().__init__(
+                f"Detected corrupted protocol row ({msg}). Please fix manually."
+            )
+
 
 ### Protocol container classes ###
 
@@ -248,7 +264,8 @@ class ProtocolDay(ProtocolMeta):
             for row in reader:
                 try:
                     record = Record.from_protocol_row(row)
-                except IOError as err:
+                    self._raise_if_invalid_for_me(record)
+                except Record.CorruptedProtocolRowError as err:
                     raise IOError(f"{err}\n  In: {self.path}")
                 entries.append(record)
 
@@ -257,6 +274,13 @@ class ProtocolDay(ProtocolMeta):
 
         return entries
 
+    def _raise_if_invalid_for_me(self, record: Record):
+        """ Check if the record matches this ProtocolDay and raise if not. """
+        if record.start.date() != self.date:
+            raise Record.CorruptedProtocolRowError("date does not match file path")
+        if record.start.date() != record.end.date():
+            raise Record.CorruptedProtocolRowError("start and end date differ")
+
     def add(self, record: Record, force: bool = False) -> None:
         """
         Store a new entry on disk. Rewrites the file.
